/** @file
 * $Source: /afs/ba.infn.it/user/pamela/src/CVS/chewbacca/PamOffLineSW/techmodel/TabDumpReader.cpp,v $
 * $Id: TabDumpReader.cpp,v 1.1.1.1 2008/09/23 07:20:27 mocchiut Exp $
 * $Author: mocchiut $
 * 
 * Implementation of the TabDumpReader class.
 */

extern "C" {
    #include "CRC.h"
}
#include "ReaderAlgorithms.h"
using namespace pamela::techmodel;

/**
 * Constructor. 
 */
TabDumpReader::TabDumpReader(void): 
  TechmodelAlgorithm(PacketType::TabDump, "TechmodelTabDump") { 
  TabDump = new TabDumpEvent();
}

/**
 * Get a string with the version info of the algorithm.
 */
std::string TabDumpReader::GetVersionInfo(void) const {
  return "$Header: /afs/ba.infn.it/user/pamela/src/CVS/chewbacca/PamOffLineSW/techmodel/TabDumpReader.cpp,v 1.1.1.1 2008/09/23 07:20:27 mocchiut Exp $\n";
}

/**
 * Initialize the algorithm with a special run. This will initialize the
 * event reader routines for all packet types.
 */
void TabDumpReader::Init(PamelaRun *run) {
  run->WriteSubPacket(this, &TabDump, TabDump->Class());
}

/**
 * Unpack the TabDump event 
 */
void TabDumpReader::PKT_RunEvent(char* subData, long int length) throw (WrongCRCException){
	std::stringstream oss; 
	string msg;	
    UINT16    subCRC;      //calculated CRC of the data
    UINT16    readCRC;     //CRC read from the end of the subpacket
    long int  dataLength = length - 2; //the block of data
    subCRC  = CM_Compute_CRC16(0, (UINT8*)subData, dataLength);
    readCRC = (((UINT16)(subData[length - 2]<<8))&0xFF00) + (((UINT16)subData[length - 1])&0x00FF);
    
    //if (subCRC != readCRC) throw WrongCRCException(" Wrong CRC for TabDump Packet ");
    if (subCRC != readCRC)
    {
    	oss.str("");
    	oss<<"Wrong CRC for TabDump Packet: "<<" CRC COMPUTED= "<< subCRC<<" CRC READ=  "<< readCRC;
    	msg=oss.str();
    	PamOffLineSW::mainLogUtil->logError(msg);	
    	throw WrongCRCException(" Wrong CRC for TabDump Packet. THE PACKET IS DISCARDED ");
    }
      
    TabDumpRecord* rec;
    TabDump->PARAMETER_STAMP = (((UINT32)subData[0]<<24)&0xFF000000) + (((UINT32)subData[1]<<16)&0x00FF0000) +  (((UINT32)subData[2]<<8)&0x0000FF00) + (((UINT32)subData[3])&0x000000FF);
    long int offset = 4;
    int i = 0;
    TabDump->Records->Clear();
    TClonesArray &recs = *(TabDump->Records);
     while (offset < dataLength){
         rec = new(recs[i++]) TabDumpRecord(); //add a new TabDump
         rec->Tab_ID  = ((UINT8)subData[offset])&0xFF;
         rec->Nrow = ((UINT8)subData[offset+1])&0xFF;
         rec->Ncol = ((UINT8)subData[offset+2])&0xFF;
         rec->Data = new TArrayI((int)((rec->Nrow)*(rec->Ncol)), (int*)(subData+offset+3));
         offset = offset + sizeof(UINT32)*(rec->Nrow)*(rec->Ncol) + 3;
     }
    
    /*
       if (subCRC != readCRC)
       {
       	oss.str("");
       	oss<<"Wrong CRC for TabDump Packet: "<<" CRC COMPUTED= "<< subCRC<<" CRC READ=  "<< readCRC;
       	msg=oss.str();
       	PamOffLineSW::mainLogUtil->logWarning(msg);	
       	throw WrongCRCException_PKTUsed(" Wrong CRC for TabDump Packet. ");
       }
 */ 	
    
}
